#ifndef EVENT_CHAPTER_ONE
#define EVENT_CHAPTER_ONE

#include "wx/wx.h"

#define USE_DYNAMIC_CONNECT

/*
    应用程序一直停留在一个循环中，等待着来自用户或者其他地方
    （比如窗口刷新或网络连接）的事件，一旦收到某种事件，应用程序就将其扔给处理这个事件的函数。虽然看上
    去不同的窗口是同时被刷新的，但实际上，绝大多数的GUI程序都是单线程的，因此窗口的刷新是依次按顺序进
    行的。如果由于某种意外你的电脑变得很慢导致窗口刷新的过程变的很明显，你就会注意到这一点。
    不同的GUI编程架构用不同的方法将它内部的事件处理机制展现给程序开发者。对于wxWidgets来说，事件表机
    制是最主要的方法。
*/

/*
事件表和事件处理过程

    wxWidgets事件处理系统比起通常的虚方法机制来说要稍微复杂一点，但它的一个好处是可以避免需要实现基类
    中所有的虚方法，因为实现所有的基类虚方法有时候是不切实际的或者是效率很低的。
    每一个wxEvtHandler的派生类，例如frame，按钮，菜单以及文档等，都会在其内部维护一个事件表，用来告诉
    wxWidgets事件和事件处理过程的对应关系。所有继承自wxWindow的窗口类，以及应用程序类都是
    wxEvtHandler的派生类.
    要创建一个静态的事件表(意味着它是在编译期间创建的)，你需要下面几个步骤：

    1. 定义一个直接或者间接继承自wxEvtHandler的类.
    2. 为每一个你想要处理的事件定义一个处理函数。
    3. 在这个类中使用DECLARE_EVENT_TABLE声明事件表。
    4. 在.cpp文件中使用使用BEGIN_EVENT_TABLE和END_EVENT_TABLE实现一个事件表。
    5. 在事件表的实现中增加事件宏，来实现从事件到事件处理过程的映射。
    
    所有的事件处理函数拥有相同的形式。他们的返回值都是void，他们都不是虚函数，他们都只有一个事件对象作
    为参数。（如果你熟悉MFC，这可能会让你觉得轻松，因为在MFC中消息处理函数并没有一个统一的形式。）这
    个事件对象的类型是随这个处理函数要处理的事件的变化而变化的。例如简单控件（比如按钮）的命令处理函数
    和菜单命令的处理函数的参数都是wxCommandEvent类型，而size事件（这个事件通常是由用户改变窗口的客
    户区尺寸而引起的）处理函数的参数则是wxSizeEvent的类型。不同的事件参数类型可以调用的方法也不相同，
    通过这些方法，你可以获得事件产生的原因以及产生这个事件的控件的值的改变情况（比如，文本框中的值的改
    变）。当然最简单的情形是你完全不需要访问这个参数的任何方法，比如按钮点击事件。
*/

/*
过滤某个事件
    wxWidgets事件处理系统实现了一些和C++中的虚方法非常类似的机制，通过这种机制，你可以通过重载某种基
    类的事件表的方法来改变基类的默认的事件处理过程。在多数情况下，通过这种方法，你甚至可以改变本地原生
    控件的默认行为。举例来说，你可以过滤某些按键事件以便本地原生的编辑框控件不处理这些按键。要达到这个
    目的，你需要实现一个继承自wxTextCtrl的新的类，然后在其事件表中使用EVT_KEY_DOWN事件映射宏。过滤
    所有的按键事件也许不是你想要的，这时候，你可以通过调用wxEvent::Skip函数来提示事件处理过程对于其中的
    某些按键事件应该继续寻找其父类的事件表。
    总的来说，在wxWidgets中，你应该通过调用事件的Skip方法，而不是通过显式直接调用其父类对应函数的方法
    来实现对特殊事件的过滤。
    下面的这个例子演示怎样让你自己的文本框控件只接受"a"到"z"和"A"到"Z"的按键，而忽略其它按键的方法：

    void MyTextCtrl::OnChar(wxKeyEvent& event)
    {
        if ( wxIsalpha(event.KeyCode()))
        {
            //这些按键在可以接受的范围，所以按照正常的流程处理
            event.Skip();
        }
        else
        {
            // 这些事件不在我们可以接受的范围，所以我们不调用Skip函数
            // 由于事件表已经匹配并且没有调用Skip函数，所以事件处理过程不会
            // 再继续匹配别的事件表，而是认为事件处理已经结束
            wxBell();
        }
    }
*/

/*
挂载事件表：
    你并不一定非要实现继承自某个类的新类，才可以改变它的事件表。对于那些继承自wxWindow的类来说，有另
    外一种可取代的方法。你可以通过实现一个新的直接继承自wxEvtHandler的新类，然后定义这个新类事件表，然
    后使用wxWindow::PushEventHandler函数将这个事件表压入到某个窗口类的事件表栈中。最后压入的那个事件
    表在事件匹配过程中将会被最先匹配，如果在其中没有匹配到对应的事件处理过程，那么栈中以前的事件表仍将
    被匹配，如此类推。你还可以用wxWindow::PopEventHandler函数来弹出最顶层的事件表，如果你给
    wxWindow:: PopEventHandler函数传递的是True的参数，那么这个弹出的事件表将被删除。
    这种方法可以避免大量的类重载，也使不同的类的实例共享同一个事件表成为可能。
    有时候，你需要手动调用窗口类的事件表，这时候你应该使用wxWindow::GetEventHandler方法，而不是直接
    使用调用这个窗口类的成员函数。虽然wxWindow::GetEventHandler通常返回这个窗口类本身。但是如果你之
    前曾经使用 PushEventHandler压入另外一个事件表，那么函数将会返回处于最顶层的事件表。因此使用
    wxWindow:: GetEventHandler函数才可以保证事件被正确的处理。
    PushEventHandler的方法通常用来临时的或者永久的改变图形界面的行为。举例来说，加入你想在你的应用程
    序实现对话框编辑的功能。你可以捕获这个对话框和它的内部控件的所有的鼠标事件，先使用你自己的事件表处
    理这些事件，来进行类似拖拽控件，改变控件大小以及移动控件动作。这在联机教学中也是很有用技术。你可以
    在你自己的事件表中过滤收到的事件，如果是可以接受的，则调用wxEvent::Skip函数正常处理。
*/

/*
动态事件处理
    前面我们讨论的事件处理方法，都是静态的事件表，这也是我们处理事件最常用的方式。接下来，我们来讨论一
    下事件表的动态处理，也就是说在运行期改变事件表的映射关系。使用这种事件映射方法的原因，可能是你想在
    程序运行的不同时刻使用不同的映射关系，或者因为你使用的那种语言(例如python)不支持静态映射，或者仅仅
    是因为你更喜欢动态映射。因为动态映射的方法可以使你更精确的控制事件表的细节，你甚至可以单独的将事件
    表中的某一个条目在运行期打开或者关闭，而前面说的PushEventHandler和PopEventHandler的方法只能针对
    整个事件表进行处理。除此以外，动态事件处理还允许你在不同的类之间共享事件函数。
    和动态事件处理相关的API有两个:wxEvtHandler::Connect和wxEvtHandler::Disconnect。大多数情况下你不需
    要手动调用wxEvtHandler::Disconnect函数，这个函数将在窗口类被释放的时候自动。
*/

class MyApp : public wxApp {
public:
    virtual bool OnInit();
};

class MainFrame : public wxFrame {
public:
    MainFrame(const wxString& title);
    void OnQuit(wxCommandEvent& event);
    void OnAbout(wxCommandEvent& event);

/*
    让我们来扩展一下前一章中的例子，来增加一个窗口大小改变事件的处理和一个确定按钮的处理。
*/
    void OnSize(wxSizeEvent& event);
    void OnButtonOK(wxCommandEvent& event);

#ifndef USE_DYNAMIC_CONNECT
private:
    DECLARE_EVENT_TABLE()
#endif
};
#endif